//==================================================================
//==================================================================

#include "common.h"

using namespace std;

//------------------------------------------------------------------
// vtest - test Vector3 class methods
//------------------------------------------------------------------

	
void test_v3(){
	cout << "----------------------------Testing Vector3 -----------------------" << endl;	
	double	a, c[3];
	bool b;
	Vector3	u, v(1), w(0,1), xz(0,1);
	Vector3 test1(2,3,4), test2(4,5,6);
	Vector3	vx[3];


	// output preset coords of v and w
	cout << "Initial u = " << u << endl;
	cout << "Initial v = " << v << endl;
	cout << "Initial w = " << w << endl;
	cout << "Initial x = " << xz << endl;
	cout << "Initial t1 = " << test1 << endl;
	cout << "Initial t2 = " << test2 << endl;
	cout << endl;


	//test getters setters
	Vector3 v3;
	v3.set(0,1);
	v3.set(1,25);
	v3.set(2,1);
	cout << "Initial v3 = " << v3 << endl;
	cout << "v3.get(1) = " << v3.get(1) << endl;

	v3[0] = 2;
	v3[1] = 3;
	v3[2] = 4;
	cout << "v3 now = " << v3 << endl;
	cout << "v3[1] = " << v3[1] << endl;
	
	

	// test operators
	b = v == v;		cout << "v == v = " << b << endl;
	b = v != v;		cout << "v != v = " << b << endl;
	b = v == w;		cout << "v == w = " << b << endl;
	b = v != w;		cout << "v != w = " << b << endl;
	u = v / 2;			cout << "v/2 = " << u << endl;
	u = v * 2;		cout << "2*v = " << u << endl;
	xz *= 2;			cout << "x *= 2 = " << xz << endl;
	u = w * 2;			cout << "w * 2 = " << u << endl;
	u = v + w;		cout << "v + w = " << u << endl;
	u = (v + w) * 2;	cout << "(v + w) * 2 = " << u << endl;
	u = v - w;		cout << "v - w = " << u << endl;
	u = (v - w) * 2;	cout << "(v - w)*2 = " << u << endl;
	u = v ^ w;		cout << "(Cross-product) v ^ w = " << u << endl;
	u = xz ^ w;		cout << "(Cross-product) x ^ w = " << u << endl;
	u = test1 ^ test2;		cout << "(Cross-product) t1 ^ t2 = " << u << endl;
	a = v * w;		cout << "(Dot-product) v * w = " << a << endl;
	a = w * xz;		cout << "(Dot-product) w * x = " << a << endl;
	cout << endl;


	a = v.len2();		cout << "v.len2() = " << a << endl;
	a = v.len();		cout << "v.len()  = " << a << endl;
	a = test1.len();		cout << "t1.len()  = " << a << endl;
	a = test1.len2();		cout << "t1.len2()  = " << a << endl;
	v.normalize();		cout << "v.normalize() = " << v << endl;
	w.normalize();		cout << "w.normalize() = " << w << endl;
	test1.normalize();		cout << "t1.normalize() = " << test1 << endl;
	test2.normalize();		cout << "t2.normalize() = " << test2 << endl;
	cout << endl;

	// check that a few identities are always true
	int cnt=0, errs=0;
	Vector3 t1, t2;
	double d1, d2;
	Vector3 u0, uinc(2,0,-2);
	Vector3 v0, vinc(1,1,1);
	Vector3 w0, winc(0.2, 0.2, 0.0);
	Vector3 x, x0(-1,-2,-3), xinc(0.5,-0.5,0.1);
	double dl, dr;		// left and right side doubles
	Vector3 vl, vr;		// left and right side Vector3s
	double er, e=0.000000000001; // allow small float error e

	for (u=u0; u[0] < 7; u+=uinc) 
	for (v=v0; v[0] < 11; v+=vinc) 
	for (w=w0; w[0] < 1.1; w+=winc) {
		++cnt;
		// Triple product
		dl = u * (v ^ w);
		dr = (u ^ v) * w;
		er = abs(dl-dr);
		if (er > e) {	// identity fails > e
			++errs;	// count errors
			cout << "Triple-Prod Error: err= " << er << endl;
		}
		++cnt;
		
		// Left Association of Cross Product
		vl = (u ^ v) ^ w;
		d1 = (u * w);
		d2 = (v * w);
		t1 = (v * d1);
		t2 = (u * d2);
		vr =  t1 - t2;
		er = (vl-vr).len();
		if (er > e) {	// identity fails > e
			++errs;	// count errors
			cout << "Left-assoc Error: err= " << er << endl;
		}
		++cnt;
		
		// Lagrange Identity
		for (x=x0; x[0] < 1.1; x+=xinc) {
			dl = (u ^ v) * (w ^ x);
			dr = (u * w) * (v * x) - (v * w) * (u * x);
			er = abs(dl-dr);
			
			if (er > e) {	// identity fails > e
				++errs;	// count errors
				cout << "Langrange Error: err= " << er << endl;
			}
			++cnt;
		}
	}
	cout << "Did " << cnt << " identity chks. Had " << errs << " errors\n";
}


//------------------------------------------------------------------
// vtest - test Vector3 class methods
//------------------------------------------------------------------

	
void test_v4(){
	cout << "----------------------------Testing Vector4 -----------------------" << endl;	
		
	double	a, c[3];
	Vector4	u, v(1), w(0,1), xz(0,1);
	Vector4 test1(2,3,4,5), test2(4,5,6,7), test3(10,11,12,13);
	//Vector4	vx[3];


	// output preset coords of v and w
	cout << "Initial v = " << v << endl;
	cout << "Initial w = " << w << endl;
	cout << "Initial x = " << xz << endl;
	cout << "Initial t1 = " << test1 << endl;
	cout << "Initial t2 = " << test2 << endl;
	cout << "Initial t3 = " << test3 << endl;
	cout << endl;


	// test operators
	u = v * 2;		cout << "2*v = " << u << endl;
	xz *= 2;			cout << "x *= 2 = " << xz << endl;
	u = w * 2;			cout << "w * 2 = " << u << endl;
	u = v + w;		cout << "v + w = " << u << endl;
	u = (v + w) * 2;	cout << "(v + w) * 2 = " << u << endl;
	u = v / 2;		cout << "v/2 = " << u << endl;
	u = v - w;		cout << "v - w = " << u << endl;
	u = (v - w) * 2;	cout << "(v - w)*2 = " << u << endl;
	cout << endl;


	a = v.len2();		cout << "v.len2() = " << a << endl;
	a = v.len();		cout << "v.len()  = " << a << endl;
	a = test1.len();		cout << "t1.len()  = " << a << endl;
	a = test1.len2();		cout << "t1.len2()  = " << a << endl;
	v.normalize();		cout << "v.normalize() = " << v << endl;
	w.normalize();		cout << "w.normalize() = " << w << endl;
	test1.normalize();		cout << "t1.normalize() = " << test1 << endl;
	test2.normalize();		cout << "t2.normalize() = " << test2 << endl;
	test3.dehomogenize();		cout << "t3.dehomogen() = " << test3 << endl;
	cout << endl;


}


void test_mat(){
	cout << "----------------------------Testing Matrxi4 -----------------------" << endl;	
	
	Matrix4	u, v(1), w(1), xz(5);
	double dub[4][4] = {{1,2,3,4},{2,3,4,5},{4,5,6,7},{6,7,8,9}};
	double *flat;
	
	Matrix4 test1(dub);
	Vector4 vec4_1(2,3,4,5), v4;
	Vector3 vec3_1(2,3,4), v3;
	

	// output preset coords of v and w
	cout << "Initial v = " << v << endl;
	cout << "Initial w = " << w << endl;
	cout << "Initial x = " << xz << endl;
	cout << "Initial t1 = " << test1 << endl;
	cout << endl;
	
	//test ops
	//mat x mat
	u = v * w;			cout << "v * w = " << u << endl;
	u = v * test1;		cout << "v * t1 = " << u << endl;

	u = test1.transpose();		cout << "t1.transpose() = " << u << endl;

	flat = test1.flatten();		cout << "t1.flatten()[0] = " << flat[0] << endl; cout << "t1.flatten()[4] = " << flat[4] << endl; cout << "t1.flatten()[7] = " << flat[7] << endl;
	flat = test1.GL_matrix();		cout << "t1.GL_matrix()[0] = " << flat[0] << endl; cout << "t1.GL_matrix()[4] = " << flat[4] << endl; cout << "t1.GL_matrix()[7] = " << flat[7] << endl;

	//mat x vec
	v4 = v * vec4_1;			cout << "v * vec4 = " << v4 << endl;
	v3 = v * vec3_1;			cout << "v * vec3 = " << v3 << endl;
	xz.make_rotX(M_PI/2);			cout << "rotX(90) = " << xz << endl;
	xz.make_rotY(M_PI/2);			cout << "rotY(90) = " << xz << endl;
	xz.make_rotZ(M_PI/2);			cout << "rotZ(90) = " << xz << endl;
	xz.make_rot(M_PI/2, M_PI/2, M_PI/2); cout << "rot(90, 90, 90) = " << xz << endl;
	xz.make_rot(M_PI/2, M_PI, M_PI/2); cout << "rot(90, 180, 90) = " << xz << endl;

	xz.make_rot(vec3_1, M_PI); cout << "rot([2,3,4], 90) = " << xz << endl;


	xz.make_scaling(2,2,2); cout << "scaling(2,2,2) = " << xz << endl;
	xz.make_translation(3,3,3); cout << "trans(3,3,3) = " << xz << endl;
	
	cout << endl;
	
	
	
}

int main()
{
	test_v3();
	test_v4();
	test_mat();
	return 0;
}
