//==================================================================
//==================================================================

#include "common.h"
using namespace std;

//==================================================================
// Vector4 Class Methods
//==================================================================


//------------------------------------------------------------------
// Comparisons
//------------------------------------------------------------------

bool Vector4::operator==( Vector4 Q)
{
	return (v[0]==Q[0] && v[1]==Q[1] && v[2]==Q[2] && v[3]==Q[3]);
	
}

bool Vector4::operator!=( Vector4 other)
{
	return !(*this == other);
}

double& Vector4::operator[](const int i){
	return v[i];
}


//------------------------------------------------------------------
// Compound assignment
//------------------------------------------------------------------
Vector4& Vector4::operator+=( Vector4 Q)        // +ve translation
{
	int i;
	for (i = 0; i<DIM4; i++){
		v[i] += Q[i];
	}
	return *this;
}

Vector4& Vector4::operator-=( Vector4 Q)        // -ve translation
{
	int i;
	for (i = 0; i<DIM4; i++){
		v[i] -= Q[i];
	}
	return *this;
}

Vector4& Vector4::operator*=( double c)        // scalar mult
{
	int i;
	for (i = 0; i<DIM4; i++){
		v[i] *= c;
	}
	return *this;
}

Vector4& Vector4::operator/=( double c)        // scalar div
{
	int i;
	for (i = 0; i<DIM4; i++){
		v[i] /= c;
	}
	return *this;
}


//------------------------------------------------------------------
// Vector4 arithmetic Operations
//------------------------------------------------------------------

Vector4 Vector4::operator+(Vector4 &other) {
    return Vector4(*this) += other;
}

Vector4 Vector4::operator-(Vector4 &other) {
    return Vector4(*this) -= other;
}

Vector4 Vector4::operator*(double other) {
    return Vector4(*this) *= other;
}

Vector4 Vector4::operator/(double other) {
    return Vector4(*this) /= other;
}


//------------------------------------------------------------------
//  Special
//------------------------------------------------------------------


void Vector4::normalize() {                      // convert to unit length
	int i;
	double ln = len();
	if (ln == 0) return;                    // do nothing for nothing

	for (i = 0; i<DIM4; i++){
		v[i] /= ln;
	}
}

void Vector4::dehomogenize()                 // make fourth component = 1
{
	int i;
	for (i = 0; i<DIM4; i++){
		v[i] /= v[3];
	}
	
}

// Write output Vector in format: "(%f, %f, %f)"
std::ostream& operator<<(std::ostream &out, Vector4 &val) {
	out << "(" << val[0] << ", " << val[1] << ", " << val[2] << ", " << val[3] << ")";
	return out;
}

