#include "common.h"

extern bool lin_hyper;

//rasterizes a given vertex to the framebuffer
//returns its x,y,z screen coords, as well as the homog. coord 'w'
Vertex2D rasterizeVertex(Vertex3D vex){
//	cout << "rastererize Vertex3D called w/ coords " << vex.coords << endl;
	Vertex2D vx2d;
	
	//promote to Vector4 with homegenous coords, w = 1
	Vector4 t(vex.coords[0], vex.coords[1], vex.coords[2], 1), new_t;
	Matrix4 m4, M;
	//cout << "t: " << t << endl;
	
	/*
	//calculate the mat M
	//M = model * cam.invertedCamMatrix() * frust.getProjectionMat() * View_Port.getVPMatrix();
	M =  View_Port.getVPMatrix() * frust.getProjectionMat() * cam.invertedCamMatrix() * model;
	cout << "M: " << M << endl;
	new_t = M * t;
	cout << "newt after M: " << new_t << endl;
	*/
	
	//take the point through the moves!!
	//first Model mat
	t = model * t;
	//cout << "t after M: " << t << endl;
	
	m4 = cam.invertedCamMatrix();
	//cout << "cam^-1: " << m4 << endl;
	
	//then camera^-1
	t = m4 * t;
	//t = cam.getCamMatrix() * t;
	
	//cout << "t after C^-1: " << t << endl;
	
	m4 = frust.getProjectionMat();
	//cout << "Proj-Mat: " << m4 << endl;
	
	//now projection
	t = frust.getProjectionMat() * t;
	//cout << "t after P: " << t << endl;
	
	
	//and last but not least: Viewport Mat
	new_t = View_Port.getVPMatrix() * t;
	//cout << "t after D: " << new_t << endl;
	
	//now we have the coords in t, get them out
	double x = ( new_t[0] / new_t[3]);
	double y = ( new_t[1] / new_t[3]);
	double z = ( new_t[2] / new_t[3]);
	//cout << "x: " << x << ", y: " << y << ", z: " << z << endl;
	
	Point2D p2d;
	p2d.x = round(x); 
	p2d.y = round(y); 
	p2d.z = z; 
	p2d.w = new_t[3]; 
	
	//and paint pixel in framebuffer
	drawPoint(p2d.x, p2d.y, p2d.z, vex.color);
//	cout << "leaving rasterize!" << endl;
	
	vx2d.color = vex.color;
	vx2d.coords = p2d;
	return vx2d;
	
}

//Proj3: Part 2, rasterizes a triangle
void rasterizeTriangle(Vertex3D vs[3])
{
//	cout << "-----------------------------------\nRasterizeTriang called!" << endl;
	int i, i_x, i_y;
	Vertex2D p_v2d[3];
	
	Point2D bb_low, bb_high;
	Point2D p, a, b, c;
	double beta, gamma, alpha, i_z;
	Color i_color;
	
	//first map all three vexs to screen coords
	//cout << "Triang screencoords: " << endl;
	for (i = 0; i < 3; i++){
		p_v2d[i] = rasterizeVertex(vs[i]);
//		cout << "triang[" << i << "] screenc: (" << p_v2d[i].coords.x << "," << p_v2d[i].coords.y << ")" << endl;
	}
	//then compute bounding box
	bb_low.x = bb_low.y = 0xFFFF;
	bb_high.x = bb_high.y = -1;

	for (i = 0; i< 3; i++){
		if (p_v2d[i].coords.x < bb_low.x)
			bb_low.x = p_v2d[i].coords.x;
		if (p_v2d[i].coords.x > bb_high.x)
			bb_high.x = p_v2d[i].coords.x;

		if (p_v2d[i].coords.y < bb_low.y)
			bb_low.y = p_v2d[i].coords.y;
		if (p_v2d[i].coords.y > bb_high.y)
			bb_high.y = p_v2d[i].coords.y;
			
	}
	
	//cout << "BBox low:(" << bb_low.x << "," << bb_low.y << "), high: (" << bb_high.x << "," << bb_high.y << ")" << endl;
	
	
	a = p_v2d[0].coords; 
	b = p_v2d[1].coords; 
	c = p_v2d[2].coords;
	
	//calculate f_ac(b) & f_ab(c)
	
	double f_ac_b = (double)f_ab(a, c, b);
	double f_ab_c = (double)f_ab(a, b, c);
	//cout << "f_ac_b:" << f_ac_b << ", f_ab_c" << f_ac_b << endl;
	
	
	//now we step through each pixel inside the bbox
	for (i_x = bb_low.x; i_x < bb_high.x; i_x++){
		for (i_y = bb_low.y; i_y < bb_high.y; i_y++){
			//cout << "testing point:(" << i_x << "," << i_y << ")" << endl;
			p.x = i_x; p.y = i_y;
			
			//first calc its barycentric coords
			beta = ( (double)f_ab(a, c, p) / f_ac_b);
			gamma = ( (double)f_ab(a, b, p) / f_ab_c);
			
			alpha = 1 - beta - gamma;
			
			//interpolate the z-value
			i_z = alpha*a.z + beta*b.z + gamma*c.z;
			
			if(!lin_hyper){
				//interpolate color linearily
				i_color = interpolate_color_lin(alpha, beta, gamma, vs[0].color, vs[1].color, vs[2].color);
			}else{
				//interpolate color hyperbolicly
				i_color = interpolate_color_hyperb(alpha, beta, gamma, p_v2d[0], p_v2d[1], p_v2d[2]);
			}
			//now test:
			//if 0 < alpha, beta, gamma < 1 it is inside triangle: draw!
			if (alpha > 0 && alpha < 1 && beta > 0 && beta < 1 && gamma > 0 && gamma < 1){
				drawPoint(p.x, p.y, i_z, i_color);
			}
		}	
	}
	
//	cout << "leaving RasterizeTriang!\n-----------------------------------" << endl;
	

}

int dot_p(Point2D a, Point2D b){
	return (a.x*b.x + a.y*b.y);
}


//get the barycentric coord from a point p and 2 points a & b
int f_ab(Point2D a, Point2D b, Point2D p){
	int ret;
	Point2D normal, line_ap;
	normal.x = a.y - b.y;
	normal.y = b.x - a.x;
	
	line_ap.x = p.x - a.x;
	line_ap.y = p.y - a.y;
	
	//now calculate and return the dot product
	ret = dot_p(normal, line_ap);
	return ret;
}

Color interpolate_color_lin(double alpha, double beta, double gamma, Color ac, Color bc, Color cc){
	
	Color c;
	c.r = alpha*ac.r + beta*bc.r + gamma*cc.r;
	c.g = alpha*ac.g + beta*bc.g + gamma*cc.g;
	c.b = alpha*ac.b + beta*bc.b + gamma*cc.b;
	
	return c;
	
}


Color interpolate_color_hyperb(double alpha, double beta, double gamma, Vertex2D a, Vertex2D b, Vertex2D c){
	
	
	Color col;
	double c_w;
	
	//compute 1/w
	double w_1 = alpha * ( 1 / a.coords.w) + beta * ( 1 / b.coords.w) + gamma * ( 1 / c.coords.w);
	
	//compute c/w
	//c/w for r
	c_w = alpha * ( a.color.r / a.coords.w) + beta * ( b.color.r / b.coords.w) + gamma * ( c.color.r / c.coords.w);
	col.r = c_w / w_1;
	//c/w for g
	c_w = alpha * ( a.color.g / a.coords.w) + beta * ( b.color.g / b.coords.w) + gamma * ( c.color.g / c.coords.w);
	col.g = c_w / w_1;
	//c/w for b
	c_w = alpha * ( a.color.b / a.coords.w) + beta * ( b.color.b / b.coords.w) + gamma * ( c.color.b / c.coords.w);
	col.b = c_w / w_1;
	
	
	return col;
	
}


Color part2b_rep_stripe_func(Color c){
	Color col;
	
	col.r = ((int)floor(10*c.r)) % 2;
	col.g = ((int)floor(10*c.g)) % 2;
	col.b = ((int)floor(10*c.b)) % 2;
	
	return col;
}
