//==================================================================
// 
// 
// 
// 
//==================================================================

#ifndef SS_Matrix4_H
#define SS_Matrix_H

//#include "common.h"

//==================================================================
//  Matrix4 Class Definition
//==================================================================

class Matrix4 {
	
protected:					//		ROW-MAJOR !!!!!!!!!!!!!
		double v[4][4];		//4 rows x 4 cols = v[row][col]      
	
public:
	//----------------------------------------------------------
	// Lots of Constructors (add more as needed)
	Matrix4() { 
		int i,j;
		for (i=0; i < MATRIX_DIM; i++)
			for (j=0; j < MATRIX_DIM; j++)
				v[i][j] = 0;
		 
	}

	// 3D Matrix4
	Matrix4( double a) {
		int i,j;
		for (i=0; i < MATRIX_DIM; i++)
			for (j=0; j < MATRIX_DIM; j++)
				v[i][j] = a;
	}

	// Array Matrix4
	Matrix4( double a[4][4]){
		int i,j;
		for (i=0; i < MATRIX_DIM; i++)
			for (j=0; j < MATRIX_DIM; j++)
				v[i][j] = a[i][j];
	}

	// Array Matrix4, OpenGL col-major to internal row-major
	Matrix4( double a[16]){
		int i,j;
		//row
		for (i=0; i < MATRIX_DIM; i++)
			//col
			for (j=0; j < MATRIX_DIM; j++)
				//v[col][row]
				v[j][i] = a[(i*4)+j];
	}

	// Vec4 col-major Matrix4, translate to internal row-major
	Matrix4( Vector4 a[4]){
		int i,j;
		//row
		for (i=0; i < MATRIX_DIM; i++)
			//col
			for (j=0; j < MATRIX_DIM; j++)
				v[j][i] = a[i][j];
	}

	// Destructor
	~Matrix4() {};
	/*
	void set_rowMajor(Vector4 a[4]){
		int i,j;
		//row
		for (i=0; i < MATRIX_DIM; i++)
			//col
			for (j=0; j < MATRIX_DIM; j++)
				v[i][j] = a[i][j];
		
	}
	*/
	//----------------------------------------------------------
	// get/set
	double &operator[](const int);
	double 	get(int i, int j){ return v[i][j]; }
	void 	set(int i, int j, double d){ v[i][j] = d; }
	void 	assign(double a[4][4]){
		int i,j;
		for (i=0; i < MATRIX_DIM; i++)
			for (j=0; j < MATRIX_DIM; j++)
				v[i][j] = a[i][j];
	}
	
	void set_to(double a){
		int i,j;
		for (i=0; i < MATRIX_DIM; i++)
			for (j=0; j < MATRIX_DIM; j++)
				v[i][j] = a;
	}
	
	double *flatten();
	double *GL_matrix();
	
	//----------------------------------------------------------
	// Comparison 
	bool operator==( Matrix4 );
	bool operator!=( Matrix4);


	//----------------------------------------------------------
	Vector3& operator*(Vector3);        // 
	Vector4& operator*(Vector4);        // 
	Matrix4 operator*(Matrix4);        // 
	Matrix4 operator*(double);        // 
	Matrix4 operator+(Matrix4);        // 
	Matrix4 operator-(Matrix4);        // 

	//compund ops
	Matrix4& operator*=(Matrix4);      // mat x mat mult
	Matrix4& operator*=(double);      // mat x mat mult
	Matrix4& operator+=(Matrix4);      // mat x mat mult
	Matrix4& operator-=(Matrix4);      // mat x mat mult
	
	//arithmetic ops
	double determinant();
	Matrix4& inverse();
	
	//----------------------------------------------------------
	// Special Operations
	friend std::ostream &operator<<(std::ostream &out, Matrix4 &mat);

	Matrix4& make_ident();

	//rotations
	Matrix4& make_rotX(double);
	Matrix4& make_rotY(double);
	Matrix4& make_rotZ(double);
	Matrix4& make_rot(double, double, double);		//x,y,z
	Matrix4& make_rot(Vector3, double);		//around vector

	Matrix4& make_scaling(double, double, double);
	Matrix4& make_translation(double, double, double);

	
	Matrix4& transpose();


};

#endif SS_Matrix4_H