#include "common.h"
#ifdef COLORFUL_HOUSE
	#include "house3.h"
#else
	#include "house.h"
#endif

static int window_width = 512, window_height = 512;
static float* pixels = new float[window_width * window_height * 3];
static double* z_buffer = new double[window_width * window_height];

using namespace std;

//the Model Matrix M
Matrix4 model;

Camera cam;
Frustrum frust;

//the viewport
Viewport View_Port;

//false = lin, true = hyper
bool lin_hyper = false;

void drawHouseOGL() {
    
	int i, j, idx,vxn;

	//draw all the triangs, there should be 20
	for (i = 0; i < 20; i++){
		idx = (i*3);
		
		glBegin(GL_TRIANGLES);
		
		//cout << "Drawing triangle #" << i << " with idx: " << idx << endl;

		//now draw each of the 3 vertices
		for (j = 0; j < 3; j++){
			vxn = (indices[idx+j]) * 3;
			glColor3f(colors[vxn], colors[vxn+1], colors[vxn+2]);
			//cout << "Drawing vertex #" << (idx+j) << ", vxn: " << vxn << "@[" << vertices[vxn] << "," << vertices[vxn+1] << "," << vertices[vxn+2] << "]" << " and color: (" << colors[vxn] << "," << colors[vxn+1] << "," << colors[vxn+2] << ")" << endl;
			//draw each vertex
	      	glVertex3f(vertices[vxn], vertices[vxn+1], vertices[vxn+2]);
		}      
		glEnd();

	}
	
	
}


void rasterizeVertex(double x, double y, double z, float r, float g, float b){
	Vector3 vec3(x, y, z);
	Color c;
	c.r = r; c.g = g; c.b = b;
	
	Vertex3D v;
	v.coords = vec3;
	v.color = c;
	rasterizeVertex(v);

}

//draws the House using our own rasterizer
void drawHouseMe() {
	//cout << "DrawHouseMe called!" << endl;


	Vertex3D triang_vs[3], vx;
	int i, j, idx,vxn;

	//draw all the triangs, there should be 20
	for (i = 0; i < 20; i++){
		idx = (i*3);
		
		
		//cout << "Drawing triangle #" << i << " with idx: " << idx << endl;

		//now draw each of the 3 vertices
		for (j = 0; j < 3; j++){
			vxn = (indices[idx+j]) * 3;
			//cout << "Drawing vertex #" << (idx+j) << ", vxn: " << vxn << "@[" << vertices[vxn] << "," << vertices[vxn+1] << "," << vertices[vxn+2] << "]" << " and color: (" << colors[vxn] << "," << colors[vxn+1] << "," << colors[vxn+2] << ")" << endl;
			//draw each vertex
			vx.coords[0] = vertices[vxn];
			vx.coords[1] = vertices[vxn+1];
			vx.coords[2] = vertices[vxn+2];
			Color c;
			c.r = colors[vxn]; c.g = colors[vxn+1]; c.b = colors[vxn+2];
			vx.color = c;
			
			//draw vertex if Part1 is defined
	      	if (PART_1)
				rasterizeVertex(vx);
			
			triang_vs[j] = vx;
		
		}
		
		//and draw triangle, if doing part2a
		if (PART_2A)
			rasterizeTriangle(triang_vs);      
	
	}

	//cout << "leaving DrawHouseMe!" << endl;
	return;

}

void drawStripedTriangleMe(){ 
	Vertex3D vxa, vxb, vxc;
	
	//draw a triangle with one white and two black vertices
	//20,-4,20, 20,-4,-20, -20,-4,-20
	//vxa 
	

}


// Clear frame buffer
void clearBuffer()
{
  Color clearColor = {0.0, 0.0, 0.0};   // clear color: black
  double clearZ = 0xFF;   // clear Z: -256
  for (int i=0; i<window_width*window_height; ++i)
  {
    pixels[i*3]   = clearColor.r;
    pixels[i*3+1] = clearColor.g;
    pixels[i*3+2] = clearColor.b;
	z_buffer[i]	 = clearZ;
  }  
}

// Draw a point into the frame buffer
void drawPoint(int x, int y, double z, float r, float g, float b)
{
  int offset = y*window_width*3 + x*3;
  int z_offset = y*window_width + x;

	//only draw if z is closer
	if (z_buffer[z_offset] > z){
		z_buffer[z_offset] = z;
	  	pixels[offset]   = r;
	  	pixels[offset+1] = g;
	  	pixels[offset+2] = b;
	}

}

// Draw a point into the frame buffer
void drawPoint(int x, int y, double z, Color c)
{	
	drawPoint(x,y,z,c.r,c.g,c.b);
}



void rasterize()
{
  	//drawHouseOGL uses the OpenGL code
	//drawHouseOGL();
	
	//this uses my code
	drawHouseMe();

}

void init(){
	
	//init the model mat to identity just in case
	model.make_ident();
	
	//init frustrum
	frust.set(60.0, 1.0, 1.0, 100.0);
	
	//set up the cam
	
	Vector3 ev(-10.0, 40.0, 40.0), dv(-5.0, 0.0, 0.0), uv(0.0, 1.0, 0.0);
	//Vector3 ev(0.0, 0.0, 40.0), dv(0.0, 0.0, 0.0), uv(0.0, 1.0, 0.0);
	cam.set(ev, dv, uv);
	
	Matrix4 m4 = cam.getCamMatrix();
	
	//cout << "Cam_mat is now: " << m4 << endl;
	
	
	//init viewport mat
	View_Port.set_dim(window_width, window_height);
	
	
	
}

// Called whenever the window size changes
void reshape(int new_width, int new_height)
{
//	cout << "reshape-func called w/ w: " << new_width << ",h: " << new_height << endl;
	
  window_width  = new_width;
  window_height = new_height;
  
  delete[] pixels;
  pixels = new float[window_width * window_height * 3];
  
  delete[] z_buffer;
  z_buffer = new double[window_width * window_height];

  frust.set_aspect((double)new_width/new_height);
  View_Port.set_dim(new_width, new_height);
	
}

void keyboard(unsigned char key, int, int)
{
//  cerr << "Key pressed: " << key << endl;

	switch(key){
		case 'l':
			cout << "Setting interpolation to linear!" << endl;
			lin_hyper = false;
			break;
		case 'h':
			cout << "Setting interpolation to Hyperbolic!" << endl;
			lin_hyper = true;
			break;
		
	}
	glutPostRedisplay();
	
}

void display()
{
	cout << "Display-func called!" << endl;
	
  clearBuffer();
  rasterize();

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // glDrawPixels writes a block of pixels to the framebuffer
  glDrawPixels(window_width, window_height, GL_RGB, GL_FLOAT, pixels);

  glutSwapBuffers();
//	cout << "leaving display-func!" << endl;

}

int main(int argc, char** argv) {
  glutInit(&argc, argv);

  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowSize(window_width, window_height);
  glutCreateWindow("CSE 167 Project 3");

  init();

  glutReshapeFunc(reshape);
  glutDisplayFunc(display);
  glutKeyboardFunc(keyboard);
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.0, 0.0, 0.0, 1.0);
  glutMainLoop();
}
