//==================================================================
//==================================================================

#include "common.h"
using namespace std;

//==================================================================
// Light Class Methods
//==================================================================


//------------------------------------------------------------------
// set stuff
//------------------------------------------------------------------


//set this as a positional light, and specify its position
void Light::setPositional(float x=0, float y=0, float z=1){
	
	//cout << "setting Position for " << OGL_light_index << endl;
	GLfloat light_position[] = { x, y, z, 1.0 };
	glLightfv(OGL_light_index, GL_POSITION, light_position);
	
}

//sets this as a directional light, and set its direction
void Light::setDirectional(float x=0, float y=0, float z=1){
	
	GLfloat light_position[] = { x, y, z, 0.0 };
	glLightfv(OGL_light_index, GL_POSITION, light_position);
	
}


void Light::setAmbient(float r=0, float g=0, float b=0, float a=1){

	GLfloat light_ambient[] = { r, g, b, a};
	glLightfv(OGL_light_index, GL_AMBIENT, light_ambient);	
	
}


void Light::setDiffuse(float r=1, float g=1, float b=1, float a=1){
	
	GLfloat light_diff[] = { r, g, b, a};
	glLightfv(OGL_light_index, GL_DIFFUSE, light_diff);	

}

void Light::setSpecular(float r=1, float g=1, float b=1, float a=1){
	
	GLfloat light_diff[] = { r, g, b, a};
	glLightfv(OGL_light_index, GL_SPECULAR, light_diff);	

}


//Spotlight
//x,y,z, exponent, cutoff
void Light::setSpot(float x=0, float y=0, float z=-1, float expo=0, float cut=180){
	
	//cout << "setting Spot for " << OGL_light_index << endl;
	
	GLfloat spot_direction[] = { x, y, z };
	glLightfv(OGL_light_index, GL_SPOT_DIRECTION, spot_direction);
	
	glLightf(OGL_light_index, GL_SPOT_EXPONENT, expo);
	glLightf(OGL_light_index, GL_SPOT_CUTOFF, cut);
	
	
}

//atenuation
//constant, lin, quadr.
void Light::setAttenuation(float con = 1.0, float lin = 0.0, float quad = 0.0){
	glLightf(OGL_light_index, GL_CONSTANT_ATTENUATION, con);
	glLightf(OGL_light_index, GL_LINEAR_ATTENUATION, lin);
	glLightf(OGL_light_index, GL_QUADRATIC_ATTENUATION, quad);
}



//------------------------------------------------------------------
//  Special
//------------------------------------------------------------------

// Write output Matrix
std::ostream& operator<<(std::ostream &out, Light &val) {
	out << "Light#";

	cout << val.LightIndex;
	cout << "(OGL:" << val.OGL_light_index;
	out << ")";
	
	return out;
}
