//==================================================================
//==================================================================

#include "common.h"

using namespace std;

//==================================================================
// Vector3 Class Methods
//==================================================================


//------------------------------------------------------------------
// Comparisons
//------------------------------------------------------------------

bool Vector3::operator==( Vector3 Q)
{
	return (v[0]==Q[0] && v[1]==Q[1] && v[2]==Q[2]);
	
}

bool Vector3::operator!=( Vector3 other)
{
	return !(*this == other);
}


double& Vector3::operator[](const int i){
	return v[i];
}

//------------------------------------------------------------------
// Compound assignment
//------------------------------------------------------------------
Vector3& Vector3::operator+=( Vector3 Q)        // +ve translation
{
	int i;
	for (i = 0; i<DIM3; i++){
		v[i] += Q[i];
	}
	return *this;
}

Vector3& Vector3::operator-=( Vector3 Q)        // -ve translation
{
	int i;
	for (i = 0; i<DIM3; i++){
		v[i] -= Q[i];
	}
	return *this;
}

Vector3& Vector3::operator*=( double c)        // scalar mult
{
	int i;
	for (i = 0; i<DIM3; i++){
		v[i] *= c;
		//cout << "doing Op*=, i: " << i << endl;
	}
	return *this;
}

Vector3& Vector3::operator/=( double c)        // scalar div
{
	int i;
	for (i = 0; i<DIM3; i++){
		v[i] /= c;
	}
	return *this;
}

// 3D Exterior Cross Product
Vector3& Vector3::operator^=( Vector3 w ) {
	double x = v[0], y = v[1], z = v[2];
	
	//cout << "doing Op^w/" <<(*this)<< ", other: " << w << endl;
		// y * o.z  - z * o.y
	v[0] = y * w[2] - z * w[1];
		//z * o.x  - x * o.z
	v[1] = z * w[0] - x * w[2];
		//x * o.y  - y * o.x
	v[2] = x * w[1] - y * w[0];
	
	//cout << "x(y * o.z  - z * o.y): " << v[0] << endl;
	//cout << "y(z * o.x  - x * o.z): " << v[1] << endl;
	//cout << "z(x * o.y  - y * o.x): " << v[2] << endl;
	
	return *this;
}


//------------------------------------------------------------------
// Vector3 arithmetic Operations
//------------------------------------------------------------------

 Vector3 Vector3::operator+( Vector3 &other)  {
    //cout << "doing Op+, other: " << other << endl;
	return Vector3(*this) += other;
}

 Vector3 Vector3::operator-( Vector3 &other)  {
    return Vector3(*this) -= other;
}

 Vector3 Vector3::operator*( double other)  {
    //cout << "doing Op*, other: " << other << endl;
	return Vector3(*this) *= other;
}

 Vector3 Vector3::operator/( double other)  {
    return Vector3(*this) /= other;
}



//------------------------------------------------------------------
//  Products
//------------------------------------------------------------------

// Inner Dot Product
double Vector3::operator*( Vector3 w ) {
	return (v[0] * w[0] + v[1] * w[1] + v[2] * w[2]);
}

//cross product
 Vector3 Vector3::operator^( Vector3 &other)  {
    return Vector3(*this) ^= other;
}

//------------------------------------------------------------------
//  Special
//------------------------------------------------------------------


void Vector3::normalize() {                      // convert to unit length
	double ln = len() ;
	if (ln == 0) return;                    // do nothing for nothing
	v[0] /= ln;
	v[1] /= ln;
	v[2] /= ln;
}

// Write output Vector in format: "(%f, %f, %f)"
std::ostream& operator<<(std::ostream &out, Vector3 &val) {
	out << "V3(" << val[0] << ", " << val[1] << ", " << val[2] << ")";
	return out;
}
