/* -------------------------------------------------------

This shader implements a point light per pixel using  the 
diffuse, specular, and ambient terms acoording to "Mathematics of Lighthing" 
as found in the book "OpenGL Programming Guide" (aka the Red Book)

AntÃ³nio Ramires Fernandes

--------------------------------------------------------- */
#define N_LIGHTS 2

varying vec4 diffuse[N_LIGHTS], ambient[N_LIGHTS], ambientGlobal;
varying vec3 normal, lightDir[N_LIGHTS], halfVector[N_LIGHTS];
varying float dist[N_LIGHTS];


void main()
{	
	int i;
	/* first transform the normal into eye space and normalize the result */
	normal = normalize(gl_NormalMatrix * gl_Normal);
	
	vec4 ecPos = gl_ModelViewMatrix * gl_Vertex;
	
	//do this for all defined light sources
	for (i = 0; i < N_LIGHTS; i++){
	
		
		/* now normalize the light's direction. Note that according to the
		OpenGL specification, the light is stored in eye space. Also since 
		we're talking about a directional light, the position field is actually 
		direction */
		vec3 aux = vec3(gl_LightSource[i].position-ecPos);
		lightDir[i] = normalize(aux);
	
		/* compute the distance to the light source to a varying variable*/
		dist[i] = length(aux);

		/* Normalize the halfVector to pass it to the fragment shader */
		halfVector[i] = normalize(gl_LightSource[i].halfVector.xyz);
	
		/* Compute the diffuse, ambient and globalAmbient terms */
		diffuse[i] = gl_FrontMaterial.diffuse * gl_LightSource[i].diffuse;
		ambient[i] = gl_FrontMaterial.ambient * gl_LightSource[i].ambient;
	
		
		
	}
	ambientGlobal = gl_LightModel.ambient * gl_FrontMaterial.ambient;
	
	gl_Position = ftransform();
	
	
} 
