/* -------------------------------------------------------

This shader implements a point light per pixel using  the 
diffuse, specular, and ambient terms acoording to "Mathematics of Lighthing" 
as found in the book "OpenGL Programming Guide" (aka the Red Book)

--------------------------------------------------------- */

#define N_LIGHTS 2

varying vec4 diffuse[N_LIGHTS];//, ambient[2];	//ambientGlobal
varying vec3 lightDir[N_LIGHTS], halfVector[N_LIGHTS];
varying vec3 normal;
varying float dist[N_LIGHTS];

//pointLight is light#0
vec4 point()
{
	vec3 n,halfV,viewV,ldir;
	float NdotL,NdotHV;
	vec4 color = (gl_LightModel.ambient * gl_FrontMaterial.ambient);
	float att;
	vec4 ambient = gl_LightSource[0].ambient * gl_FrontMaterial.ambient;
	
	/* a fragment shader can't write a verying variable, hence we need
	a new variable to store the normalized interpolated normal */
	n = normalize(normal);
	
	/* compute the dot product between normal and ldir */
	NdotL = max(dot(n,normalize(lightDir[0])),0.0);

	if (NdotL > 0.0) {
	
		att = 1.0 / (gl_LightSource[0].constantAttenuation +
				gl_LightSource[0].linearAttenuation * dist[0] +
				gl_LightSource[0].quadraticAttenuation * dist[0] * dist[0]);
			color += att * (diffuse[0] * NdotL + ambient);
	
		
		halfV = normalize(halfVector[0]);
		NdotHV = max(dot(n,halfV),0.0);
		color += att * gl_FrontMaterial.specular * gl_LightSource[0].specular * pow(NdotHV,gl_FrontMaterial.shininess);
	}

	return color;
}


//spotLight is light#1
vec4 spot()
{
	
	vec3 n, halfV;
	float NdotL, NdotHV;
	vec4 color = (gl_LightModel.ambient * gl_FrontMaterial.ambient);
	float att, spotEffect;
	vec4 ambient = gl_LightSource[1].ambient * gl_FrontMaterial.ambient;
	
	/* a fragment shader can't write a verying variable, hence we need
	a new variable to store the normalized interpolated normal */
	n = normalize(normal);
	
	/* compute the dot product between normal and ldir */
	NdotL = max(dot(n,normalize(lightDir[1])),0.0);

	if (NdotL > 0.0) {
	
		spotEffect = dot(normalize(gl_LightSource[1].spotDirection), normalize(-lightDir[1]));
		
		if (spotEffect > gl_LightSource[1].spotCosCutoff) {
			spotEffect = pow(spotEffect, gl_LightSource[1].spotExponent);
			att = spotEffect / (gl_LightSource[1].constantAttenuation +
					gl_LightSource[1].linearAttenuation * dist[1] +
					gl_LightSource[1].quadraticAttenuation * dist[1] * dist[1]);
				
				color += att * (diffuse[1] * NdotL + ambient);
		
			
			halfV = normalize(halfVector[1]);
			NdotHV = max(dot(n,halfV),0.0);
			color += att * gl_FrontMaterial.specular * gl_LightSource[1].specular * pow(NdotHV,gl_FrontMaterial.shininess);
		}
	}

	return color;
}


void main()
{

	gl_FragColor = point() + spot();

}