// Fragment shader for per-pixel phong shading. The
// teapot is a good object to test this shader, since
// the .obj file includes surface normals.

// The shader computes the dot product between the unit
// surface normal and light direction, which were 
// passed as varying inputs from the vertex shader. The
// result is multiplied with the vertex color, which is 
// accessed through a pre-defined varying variable.

varying vec3 N;
varying vec3 v;


vec4 spot(int lightIdx){
	float atten;
	vec4 col;
	vec4 Iamb, Idiff, Ispec;
	vec3 E;
	
		vec3 L = normalize(gl_LightSource[lightIdx].position.xyz - v); 
		vec3 R = normalize(-reflect(L,N)); 
		float dotty = max(dot(-L, gl_LightSource[lightIdx].spotDirection), 0.0);
		
		if (  dotty <= gl_LightSource[lightIdx].spotCosCutoff)
			atten = 0.0;
		else
			atten = pow(dotty, gl_LightSource[lightIdx].spotExponent);
			
		//calculate Ambient Term:
		Iamb = gl_FrontLightProduct[lightIdx].ambient * atten;

		//calculate Diffuse Term:
		Idiff = gl_FrontLightProduct[lightIdx].diffuse * max(dot(N,L), 0.0) * atten;

		// calculate Specular Term:
		Ispec = gl_FrontLightProduct[lightIdx].specular 
		                  * pow(max(dot(R,E),0.0),gl_FrontMaterial.shininess) * atten;

		// add to Total Color:
		col = Iamb + Idiff + Ispec;
		return col;
		
}

vec4 point(int lightIdx){

	vec4 Iamb, Idiff, Ispec;
	vec4 col;
	float atten;

	vec3 E = normalize(-v); // we are in Eye Coordinates, so EyePos is (0,0,0)
	vec3 L = normalize(gl_LightSource[lightIdx].position.xyz - v); 
	vec3 R = normalize(-reflect(L,N)); 
	
	
	atten = pow(length(gl_LightSource[lightIdx].position.xyz - v),2.0);
	//atten = 1.0;
	
	//calculate Ambient Term, attenuated with distance^2
	vec4 cl_amb = gl_LightSource[lightIdx].ambient / atten;
	 Iamb = cl_amb * gl_FrontMaterial.ambient;

	//calculate Diffuse Term:
	vec4 cl_dif = gl_LightSource[lightIdx].diffuse / atten;
	 Idiff = (cl_dif * gl_FrontMaterial.diffuse * max(dot(N,L), 0.0));

	// calculate Specular Term:
	vec4 cl_spec = gl_LightSource[lightIdx].specular / atten;
	 Ispec = (cl_spec * gl_FrontMaterial.specular 
	                  * pow(max(dot(R,E),0.0),gl_FrontMaterial.shininess));
	
	col = (Iamb + Idiff + Ispec);
	
	return col;
	
}

void main (void)
{
	//now do this twice for each light source once
	vec4 col;
	
	col = point(0);
	col += spot(1);

	
	
	gl_FragColor = gl_FrontLightModelProduct.sceneColor + col; 
	
}

