#include "common.h"
#define ROT_STEP 5
#define TRANS_STEP 1

using namespace std;

int Window::width  = 512;   // set window width in pixels here
int Window::height = 512;   // set window height in pixels here
bool Window::pointOn = false;   // set window height in pixels here
bool Window::spotOn = false;   // set window height in pixels here
bool use_myShader = true;
Shader myShaders;

Material mat;
Light pointLight(0), spotLight(1);

//the coords of the point light, in polar coords
//lats, elevation, inclenation
double point_phi = 180;
//longs, azimuth
double point_theta  = 90;
double point_r = 20;

double spot_x = -8, spot_y = 0, spot_z = 20;


//----------------------------------------------------------------------------
// Callback method called when window is resized.
void Window::reshapeCallback(int w, int h)
{
  width = w;
  height = h;
  glViewport(0, 0, w, h);  // set new viewport size
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-10.0, 10.0, -10.0, 10.0, 10, 1000.0); // set perspective projection viewing frustum

  /* moved camera position to modelview matrix */
  //glTranslatef(0, 0, -20);

}

//----------------------------------------------------------------------------
// Callback method called when special key is pressed.
void Window::specialFunc(int k, int x, int y)
{
	
    switch(k) {
	      case GLUT_KEY_UP:    point_phi += ROT_STEP; break;
	      case GLUT_KEY_DOWN:  point_phi -= ROT_STEP; break;
	      case GLUT_KEY_LEFT:  point_theta  += ROT_STEP; break;
	      case GLUT_KEY_RIGHT: point_theta  -= ROT_STEP; break;
    }
    
	glutPostRedisplay();
	
}

void Window::keyboardFunc(unsigned char key, int, int)
{
//  cout << "Key pressed: " << key << endl;

	switch(key){
		case 'p':
			pointOn = !pointOn;
			break;
		case 's':
			spotOn = !spotOn;
			break;
		case 'g':
			use_myShader = !use_myShader;
			
			//set shaders
			if (use_myShader){
		  		cout << "Using my shader!" << endl;
				myShaders.bind();
			}else{
				cout << "Using OpenGL's shader!" << endl;
				myShaders.unbind();
			}
			break;
		//moves spot light up/down
		case 'i':
			spot_y += TRANS_STEP;
			break;
		case 'k':
			spot_y -= TRANS_STEP;
			break;
		//left/right
		case 'j':
			spot_x -= TRANS_STEP;
			break;
		case 'l':
			spot_x += TRANS_STEP;
			break;
		//closer/further
		case 'u':
			spot_z -= TRANS_STEP;
			break;
		case 'o':
			spot_z += TRANS_STEP;
			break;
	
	}
	glutPostRedisplay();
	
}


//----------------------------------------------------------------------------
// Callback method called when window readraw is necessary or
// when glutPostRedisplay() was called.
void Window::displayCallback(void)
{
  Matrix4 sphere1,sphere2;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  // clear color and depth buffers
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  GLfloat params[4] = {1., 1., 1., 1.};
  glMaterialfv(GL_FRONT, GL_SPECULAR, params);

  /* set the camera position */
  glTranslatef(0, 0, -20);

  sphere1.make_translation(-8.0, 0, 0);
  glMultMatrixd(sphere1.GL_matrix());
  /* Switch to a finer tessellation for debugging to see the fixed function lighting
  ** be sure to turn this back to 12,12 before grading.
  glutSolidSphere(6.0, 120, 120);
  */

	//set this one shiny and diffuse
	mat.setDiffuse(0.1, 0.5, 0.8, 1.0);
	mat.setAmbient(0,0,0,1);
	mat.setSpecular(1, 1, 1, 1, 50);
	//set emissive to 0
	mat.setEmissive(0,0,0,1);
	
  glutSolidSphere(6.0, 12, 12); 
  
  // done with this object, clear the modelview matrix
  glLoadIdentity();

  /* set the camera position again since we just cleared everything */
  glTranslatef(0, 0, -20);

  sphere2.make_translation(8.0, 0, 0);
  glMultMatrixd(sphere2.GL_matrix());
  /* Switch to a finer tessellation for debugging to see the fixed function lighting
  ** be sure to turn this back to 12,12 before grading.
  glutSolidSphere(6.0, 120, 120);
  */


	//set this one only diffuse
	mat.setDiffuse(0.1, 0.5, 0.8, 1.0);
	mat.setAmbient(0,0,0,1);
	//these calls set the props to 0
	mat.setSpecular(0,0,0,1,0);
	mat.setEmissive(0,0,0,1);
	
  glutSolidSphere(6.0, 12, 12); 

glLoadIdentity();

//now let it be light!
	//set the pos of the point light
	//calc the coords
	float x = point_r * sin(deg2rad(point_theta)) * cos(deg2rad(point_phi));     // eye x-, y-, and z-coordinates
  	float y = point_r * sin(deg2rad(point_theta)) * sin(deg2rad(point_phi));
  	float z = point_r * cos(deg2rad(point_theta));
	//now set to point light
	pointLight.setPositional(x,y,z);
	spotLight.setDirectional(spot_x,spot_y,spot_z);
	
	if (pointOn){
		pointLight.enable();
	}else{
		pointLight.disable();
	}
	
	if (spotOn){
		spotLight.enable();
	}else{
		spotLight.disable();
	}
		
  glutSwapBuffers();
}




/* ---------------------------------------------------------
*
* ---------------------------------------------------------
*/

int main(int argc, char *argv[])
{
	
  GLfloat mat_specular[]   = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat mat_shininess[]  = { 50.0 };
  GLfloat light0_position[] = { 10.0, 10.0, 10.0, 0.0 };
  
  glutInit(&argc, argv);      	      	      // initialize GLUT
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);   // open an OpenGL context with double buffering, RGB colors, and depth buffering
  glutInitWindowSize(Window::width, Window::height);      // set initial window size
  glutCreateWindow("Two Spheres");    	                // open window and set window title

  glEnable(GL_DEPTH_TEST);            	      // enable depth buffering
  glClearColor(0.0, 0.0, 0.0, 0.0);   	      // set clear color to black
  glMatrixMode(GL_PROJECTION); 
  
	glEnable(GL_NORMALIZE);
	glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);

  // Generate material properties:
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
  glEnable(GL_LIGHTING);
	

  // Generate light sources:
	//light0 point: white
	pointLight.enable();
	//pointLight.setPositional(1,30,1);
	pointLight.setAmbient(0,0,0,1);
	pointLight.setDiffuse(1,1,1,1);
	pointLight.setSpecular(1,1,1,1);
	
	
	
	//light1 spot: reddish
	spotLight.enable();
	//position
	spotLight.setDirectional(spot_x,spot_y,spot_z);
	//slightly reddish
	spotLight.setAmbient(0.2,0.2,0.2,1);
	spotLight.setDiffuse(1,0.5,0.5,1);
	spotLight.setSpecular(1,0.5,0.5,1);
	//x,y,z of direction,expo,cutoff
	spotLight.setSpot(0, 0, -1, 0.5, 45);
	spotLight.setAttenuation(0.8, 0.0, 0.0);


	//init shaders
	//Shader myShaders("simple.vert", "simple.frag");
	//Shader myShaders("diffuse_shading.vert", "diffuse_shading.frag");
	//myShaders.setFiles("pointlight.vert", "pointlight.frag");
	//myShaders.setFiles("spotlight.vert", "spotlight.frag");
	myShaders.setFiles("combined.vert", "combined.frag");
	
	
	
  // Install callback functions:
  glutDisplayFunc(Window::displayCallback);
  glutReshapeFunc(Window::reshapeCallback);
  glutKeyboardFunc(Window::keyboardFunc);
  glutSpecialFunc(Window::specialFunc);
  glutMainLoop();
  return 0;
}

