#include "common.h"

#define FIXED_CAM 0

#define ROT_STEP 1
#define ROTATE 1
#define CYLINDER 1
#define MY_CYL_CODE 1

#define QUAD_COL 0
   
  double user_theta  = 0;
  double user_height = 0;
  double user_r = 10;
  float c_RotAngle = 0;

	//lats = theta
	//longs = phi

void drawSphere(double, int, int);

  void drawSphere(double r, int subs) {
	
	drawSphere(r, ceil(sqrt(subs)), ceil(sqrt(subs)));
  }

  void drawSphere(double r, int lats, int longs) {
      int i, j;
      for(i = 0; i <= (lats); i++) {
          
		  //latitude of lower corner of strip, [0 .. PI]
		  double lat0 = M_PI * ((double) (i - 1) / lats);
		  double z0  = r * cos(lat0);
          double zr0 = r * sin(lat0);
   
		  //upper corner of strip, [0 .. PI]
          double lat1 = M_PI * ((double) i / lats);
		  double z1 = r * cos(lat1);
          double zr1 = r * sin(lat1);
   

          glBegin(GL_QUAD_STRIP);

          for(j = 0; j <= (longs); j++) {
          	  
			  //longitute of current corner, [0 .. 2PI]
    		  double lng = 2 * M_PI * (double) (j - 1) / longs;
              double x = sin(lng);
              double y = cos(lng);

			  //set the color of this vertex
			  glColor3f( sin(((1.0/lats)*i)*M_PI), sin(((1.0/longs)*j)*M_PI), (1.0 / longs / lats) * j * i);
			
              glNormal3f(x * zr0, y * zr0, z0);
              glVertex3f(x * zr0, y * zr0, z0);
              
			  glNormal3f(x * zr1, y * zr1, z1);
              glVertex3f(x * zr1, y * zr1, z1);
          }
          glEnd();
      }
  }



  //topR should always be smaller for a cone!
  void drawCylinder(float baseR, float topR, float len, int slices, int stacks){
	
	//delta by which the radius is decreased for each stack
	float r_dec = (baseR - topR) / stacks;
	float r = baseR;
	
	//texture coords: we map u to the length of the cyl, v to the circumf
	//float delta_u = 1 / stacks;
	//float delta_v = 1 / slices;
	
	for(int i=1; i<stacks; i++) {
		
		 //height of lower corner of strip, [0 .. len]
		  double h0 = len * ((double) (i - 1) / stacks);
		 //radius of lower corner of strip
		  double r0 = r;
		 
		  //height upper corner of strip, [0 .. len]
          double h1 = len * ((double) i / stacks);
		//radius for upper corner is a little smaller
		  double r1 = (r - r_dec);
		
		float u0 = (double) (i-1) / stacks;
		float u1 = (double) i / stacks;
		
		
		glBegin(GL_QUAD_STRIP);

          for(int j = 0; j <= (slices); j++) {
          	  
			float v = (double) j / slices;
			
			  //theta of current corner, [0 .. 2PI]
    		  double theta = 2 * M_PI * (double) (j - 1) / slices;
              double x = cos(theta);
              double y = sin(theta);

			  
              //lower vertex
			  glTexCoord2f(u0,v); 
			  glNormal3f(x * r0, y * r0, h0);
              glVertex3f(x * r0, y * r0, h0);
              
			  //upper vertex
			  glTexCoord2f(u1,v); 
			  glNormal3f(x * r1, y * r1, h1);
              glVertex3f(x * r1, y * r1, h1);
          }
          glEnd();	
		
		  r -= r_dec;
	
	}
	
	
}

GLuint TexID1;      // Handles to our texture 

void initTexture(){
	glGenTextures(1,&TexID1);            // Allocate space for texture 
	glBindTexture(GL_TEXTURE_2D,TexID1); // Set our Tex handle as current 
	
	unsigned char * tex_data;
	int w = 512, h = 256;
	tex_data = loadPPM("ucsd_aerial.ppm", w, h);
	
	glTexImage2D(GL_TEXTURE_2D,0,3,512,256,0,GL_RGB,GL_UNSIGNED_BYTE,tex_data);
	
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glEnable(GL_TEXTURE_2D);
	
}


void GL_solidCone(GLdouble base, GLdouble height, GLint slices, GLint stacks)

   {
   
     GLUquadricObj* quadric = gluNewQuadric();
	gluQuadricTexture(quadric, GL_TRUE);
	gluQuadricOrientation(quadric, GLU_OUTSIDE);
     gluQuadricDrawStyle(quadric, GLU_FILL);
     gluCylinder(quadric, base, base, height, slices, stacks);
     gluDeleteQuadric(quadric);
 
 }
  void computeLocation1() {
      double x = user_r * cos(user_theta);     // my x-, y-, and z-coordinates
      double y = user_r * sin(user_theta);
      double z = user_height;
      double d = sqrt(x * x  + y * y + z * z); // distance to origin
   
      glMatrixMode(GL_PROJECTION); 
	  glLoadIdentity();
	  // fovy,asp,near,far
	  gluPerspective(80.0, 1.0, 1.0, 100.0); 
	  
	//glFrustum(-d * 0.5, d * 0.5, -d * 0.5, d * 0.5, d - 1.1, d + 1.1);
      glMatrixMode(GL_MODELVIEW); 
	  glLoadIdentity(); 
	  gluLookAt(x, y, z,  0, 0, 0, 0, 1, 0);
  }


void computeLocation2() {
	//fixed cam
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	// fovy,asp,near,far
	gluPerspective(80.0, 1.0, 1.0, 100.0); 
	glMatrixMode(GL_MODELVIEW); 
	glLoadIdentity();
	if (CYLINDER) 
		gluLookAt(	5.0, 5.0, 15.0, 
				0.0, 0.0, 0.0, 
				0.0, 1.0, 0.0);
   else
		gluLookAt(	0.0, 0.0, 5.0, 
				0.0, 0.0, 0.0, 
				0.0, 1.0, 0.0);
	
}

void drawQuad(){
	glBegin(GL_QUADS);
		//red, lower left
    	//glTexCoord2f(0.0,0.0); 
		glTexCoord2f(0,1); 
		if (QUAD_COL)
			glColor3f(1, 0, 0);
		glVertex3f( -1.0, -1.0, 1.0);
	    
		//blue, upper left
	    //glTexCoord2f(1.0,0.0); 
		glTexCoord2f(0,0); 
		if (QUAD_COL)
			glColor3f(0, 0, 1);
		glVertex3f( -1.0, 1.0, 1.0);
		
		//green upper right
		//glTexCoord2f(1.0,1.0); 
		glTexCoord2f(1,0); 
		if (QUAD_COL)
			glColor3f(0, 1, 0);
		glVertex3f(1.0, 1.0, 1.0);
		
		
		//white lower right
		//glTexCoord2f(0.0,1.0); 
		glTexCoord2f(1,1); 
		if (QUAD_COL)
			glColor3f(1, 1, 1);
		glVertex3f(1.0, -1.0,1.0);
	
	glEnd();
}

const GLfloat DefaultAxisLength=4;

void drawReferenceAxis()
						// Draw the reference axis
{
	
	 glPushAttrib(GL_ALL_ATTRIB_BITS);
						// Pushes attributes like current color to attribute stack.
	 glBegin(GL_LINES);
						// X axis red
      glColor3f(1, 0, 0);
      glVertex3f(0, 0, 0);
      glVertex3f(DefaultAxisLength, 0, 0);
						// Y axis green
      glColor3f(0, 1, 0);
      glVertex3f(0, 0, 0);
      glVertex3f(0, DefaultAxisLength, 0);
						// Z axis blue
      glColor3f(0, 0, 1);
      glVertex3f(0, 0, 0);
      glVertex3f(0, 0, DefaultAxisLength);
	  glEnd();
	  glPopAttrib();
						// // Pops attributes like current color from the attribute stack and sets as current.
	
}

  // Initializes information for drawing within OpenGL.
  void init() {
      GLfloat sun_direction[] = { 0.0, 2.0, -1.0, 1.0 };
      GLfloat sun_intensity[] = { 0.7, 0.7, 0.7, 1.0 };
      GLfloat ambient_intensity[] = { 0.3, 0.3, 0.3, 1.0 };
   
      glClearColor(0.0, 0.0, 0.0, 0.0);   // Set window color to black.
      if (FIXED_CAM)
	  	computeLocation2();
      else
		computeLocation1();
		
      glEnable(GL_DEPTH_TEST);            // Draw only closest surfaces
   
      glEnable(GL_LIGHTING);              // Set up ambient light.
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
   
      glEnable(GL_LIGHT0);                // Set up sunlight.
      glLightfv(GL_LIGHT0, GL_POSITION, sun_direction);
      glLightfv(GL_LIGHT0, GL_DIFFUSE, sun_intensity);
   
      glEnable(GL_COLOR_MATERIAL);        // Configure glColor().
      glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
  }
   
  // Draws the current image.
  void draw() {
      	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clear window.
      	
		//set global color to white
	  	glColor3f(1.0, 1.0, 1.0);
      	glShadeModel(GL_SMOOTH);
      
		//drawSphere(1.0, 50, 50); 
		//glutSolidSphere(1.0, 10, 10);
		//glColor3f(1.0, 0.0, 0.0);
		
		
		if (CYLINDER){
			//rotate sloooowly
			glPushMatrix();
			c_RotAngle += ROT_STEP;
			//if (c_RotAngle > 360) c_RotAngle -= 360;
			if (ROTATE)
				glRotatef(c_RotAngle, 0,0,1);
		
			if (!MY_CYL_CODE)
				GL_solidCone(2.0, 4.0, 50, 50);
			else{
				glPushMatrix();
				glRotatef(180, 1,0,0);
				glTranslatef(0,0,-2);
				
				drawCylinder(1.0, 1.0, 4*M_PI, 50, 50);
				glPopMatrix();
			}
			glPopMatrix();
		}else
			drawQuad();
		
		//drawSphere(1.0, 1000); // glutSolidSphere(1.0, 10, 10);
		drawReferenceAxis();
      	glutSwapBuffers();
  }
   
  // Arranges that the window will be redrawn roughly every 100 ms.
  void idle() {
      static int lastTime = 0;                // time of last redraw
      int time = glutGet(GLUT_ELAPSED_TIME);  // current time
   
      if(lastTime == 0 || time >= lastTime + 100) {
          lastTime = time;
          glutPostRedisplay();
      }
  }
   
  // When window becomes visible, we want the window to
  // continuously repaint itself.
  void visible(int vis) {
      glutIdleFunc(vis == GLUT_VISIBLE ? idle : NULL);
  }
   
  // Called when a "special" key is pressed
  void special(int k, int x, int y) {
      switch(k) {
	      case GLUT_KEY_UP:    user_height += 0.1; break;
	      case GLUT_KEY_DOWN:  user_height -= 0.1; break;
	      case GLUT_KEY_LEFT:  user_theta  += 0.1; break;
	      case GLUT_KEY_RIGHT: user_theta  -= 0.1; break;
      }
      if (FIXED_CAM)
	  	computeLocation2();
      else
		computeLocation1();
      
	  glutPostRedisplay();
  }
   

  int main(int argc, char **argv) {
      glutInit(&argc, argv);
      glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
      glutInitWindowPosition(50, 100);    // Set up display window.
      glutInitWindowSize(600, 600);
      glutCreateWindow("Textured cone");
   
      init();
	  initTexture();
      glutDisplayFunc(draw);
      glutVisibilityFunc(visible);
      glutSpecialFunc(special);
      glutMainLoop();
      return 0;
  }
