#include "common.h"
#define STEP 5
#define R_STEP 1


using namespace std;

int Window::width  = 512;   // set window width in pixels here
int Window::height = 512;   // set window height in pixels here
double user_phi = 45;
//lats, elevation, inclenation
double user_theta  = 20;
double user_r = 12;
GLuint TexID1,TexID2;      // Handles to our textures


//----------------------------------------------------------------------------
// Callback method called when window is resized.
void Window::reshapeCallback(int w, int h)
{
  width = w;
  height = h;
  glViewport(0, 0, w, h);  // set new viewport size

}

//----------------------------------------------------------------------------
// Callback method called when special key is pressed.
void Window::specialFunc(int k, int x, int y)
{
	
    switch(k) {
	      case GLUT_KEY_UP:    user_phi += STEP; break;
	      case GLUT_KEY_DOWN:  user_phi -= STEP; break;
	      case GLUT_KEY_LEFT:  user_theta  += STEP; break;
	      case GLUT_KEY_RIGHT: user_theta  -= STEP; break;
	  }
    Window::computeLocation();
    glutPostRedisplay();
	
}

void Window::keyboardFunc(unsigned char key, int, int)
{
  cout << "Key pressed: " << key << endl;

	switch(key){
		case 't':
			user_r += R_STEP;
			break;
		case 'g':
			user_r -= R_STEP;
			break;
		
	}
	Window::computeLocation();
	glutPostRedisplay();
	
}

void Window::computeLocation(){

		double x = user_r * sin(deg2rad(user_theta)) * cos(deg2rad(user_phi));     // eye x-, y-, and z-coordinates
		double y = user_r * sin(deg2rad(user_theta)) * sin(deg2rad(user_phi));
		double z = user_r * cos(deg2rad(user_theta));


		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60.0, 1.0, 1.0, 100.0); 

	  	//and camera
		glMatrixMode(GL_MODELVIEW); 
		glLoadIdentity();
		gluLookAt(	x, y, z, 
					0.0, 0.0, 0.0, 
					0.0, 1.0, 0.0);


}

#define AXIS_LENGTH 4
/****************************************************************************************/
void Window::drawReferenceAxis()
{
	 glPushAttrib(GL_ALL_ATTRIB_BITS);
						// Pushes attributes like current color to attribute stack.
	 glBegin(GL_LINES);
						// X axis red
      glColor3f(1, 0, 0);
      glVertex3f(0, 0, 0);
      glVertex3f(AXIS_LENGTH, 0, 0);
						// Y axis green
      glColor3f(0, 1, 0);
      glVertex3f(0, 0, 0);
      glVertex3f(0, AXIS_LENGTH, 0);
						// Z axis blue
      glColor3f(0, 0, 1);
      glVertex3f(0, 0, 0);
      glVertex3f(0, 0, AXIS_LENGTH);
	  
	glEnd();
	  glPopAttrib();
						// // Pops attributes like current color from the attribute stack and sets as current.
}

/******************************************************************************/

//----------------------------------------------------------------------------
// Callback method called when window readraw is necessary or
// when glutPostRedisplay() was called.
void Window::displayCallback(void)
{
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  // clear color and depth buffers
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//set global color to white
  	glColor3f(1.0, 1.0, 1.0);
  	glShadeModel(GL_SMOOTH);
  
	Window::computeLocation();
	//test_bezier1();
	//test_bezierVertices();
	glBindTexture(GL_TEXTURE_2D,TexID1);
	draw_bezierSurface();
	
	Window::drawReferenceAxis();
	
	//glutSolidSphere(1.2, 20, 20);

	glutSwapBuffers();
}


void Window::init(){
	// Initializes information for drawing within OpenGL.
	
      GLfloat sun_direction[] = { 0.0, 0.0, 1.0, 0.0 };
      GLfloat sun_intensity[] = { 0.7, 0.7, 0.7, 1.0 };

      GLfloat light_pos[] = { 0.0, 0.0, -20.0, 1.0 };
      GLfloat light_intensity[] = { 1, 1, 1, 1.0 };

      GLfloat ambient_intensity[] = { 0.3, 0.3, 0.3, 1.0 };
	  glEnable(GL_DEPTH_TEST);            // Draw only closest surfaces

      glEnable(GL_LIGHTING);              // Set up ambient light.
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);

      glEnable(GL_LIGHT0);                // Set up sunlight.
      glLightfv(GL_LIGHT0, GL_POSITION, sun_direction);
      glLightfv(GL_LIGHT0, GL_DIFFUSE, sun_intensity);

      glEnable(GL_LIGHT1);                // Set up sunlight.
      glLightfv(GL_LIGHT1, GL_POSITION, light_pos);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, light_intensity);

      glEnable(GL_COLOR_MATERIAL);        // Configure glColor().
      glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);


}


GLuint Window::LoadTexture(char *TexName)
  {

   TGAImg Img;        // Image loader
   GLuint Texture;
 
  // Load our Texture
   if(Img.Load(TexName)!=IMG_OK)
    return -1;
 
   glGenTextures(1,&Texture);            // Allocate space for texture
   glBindTexture(GL_TEXTURE_2D,Texture); // Set our Tex handle as current
 
   // Create the texture
    if(Img.GetBPP()==24)
     glTexImage2D(GL_TEXTURE_2D,0,3,Img.GetWidth(),Img.GetHeight(),0,
                  GL_RGB,GL_UNSIGNED_BYTE,Img.GetImg());
    else if(Img.GetBPP()==32)
     glTexImage2D(GL_TEXTURE_2D,0,4,Img.GetWidth(),Img.GetHeight(),0,
                  GL_RGBA,GL_UNSIGNED_BYTE,Img.GetImg());
    else
     return -1;
 
 
   return Texture;
  
}

void initTextures(){
	
	TexID1 = Window::LoadTexture("tex/wood.tga");
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glEnable(GL_TEXTURE_2D);
	
}

/* ---------------------------------------------------------
*
* ---------------------------------------------------------
*/

int main(int argc, char *argv[])
{
	
  
  glutInit(&argc, argv);      	      	      // initialize GLUT
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);   // open an OpenGL context with double buffering, RGB colors, and depth buffering
  glutInitWindowSize(Window::width, Window::height);      // set initial window size
  glutCreateWindow("Project5");    	                // open window and set window title

  glEnable(GL_DEPTH_TEST);            	      // enable depth buffering
  glClearColor(0.0, 0.0, 0.0, 0.0);   	      // set clear color to black
//  glMatrixMode(GL_PROJECTION); 
  
	
  Window::init();
  initTextures();
  Window::computeLocation();
  make_bezierSurface();
	
  // Install callback functions:
  glutDisplayFunc(Window::displayCallback);
  glutReshapeFunc(Window::reshapeCallback);
  glutKeyboardFunc(Window::keyboardFunc);
  glutSpecialFunc(Window::specialFunc);
  glutMainLoop();
  return 0;
}

