//==================================================================
//==================================================================

#include "common.h"
using namespace std;

#define NORMALS 1
#define COLORFUL 0


/*	----------------------------------------------------------
*	Create a bezier surface patch
*	---------------------------------------------------------
*/



/*	----------------------------------------------------------
*	Calculate line segments using Bernstein Polys
*	---------------------------------------------------------
*/

//returns a piecewise bezier curve, that is an array of vertices for it 
//n is the number of bezier segments, so we have 3N+1 control points from p0-p3N in the array
//n_points is number of points to evaluate along curve, ie. vertices to calculate
//shoud be evenly divisible by n
Point2D *piecewiseBernsteinCalc(int n, Point2D CtrlPoints[], int n_points){

	//the number of points per line segment, that is each piece of the curve
	//subtract 1 to make sure that we run t from 0..1 including 1
	int PsPerPiece = (n_points-1) / n;
	Point2D currCtrlPs[4];
	
	//Point2D vertices[n_points];
	Point2D *vertices = (Point2D*)malloc(sizeof(Point2D) * n_points);
	
	//the running param u: 0<u<3n
	double u, t, nextX = 0;
	int i,j, nextIdx = 0;
	
	//now evaluate each line segment
	for ( i = 0; i < n; i++ ){
		
		currCtrlPs[0] = CtrlPoints[i*3];
		currCtrlPs[1] = CtrlPoints[i*3 + 1];
		currCtrlPs[2] = CtrlPoints[i*3 + 2];
		currCtrlPs[3] = CtrlPoints[i*3 + 3];
		
		//because we run j from 0..psPerPiece, t now runs from 0..1
		//and each point along that segment
		for (j = 0; j <= PsPerPiece; j++){
			//the running t 0..1
			t = (double) j / PsPerPiece;
			
			//the coord
			vertices[nextIdx] = bernsteinCalculat0r(t, currCtrlPs);
			nextIdx++;
		} 
	}
	
	return vertices;
}



//cubic 2D bernstein calculat0r
//gets an array of the 4 control points p0-p4 in the xy-plane and the param t
//and returns x(t)
Point2D bernsteinCalculat0r(double t, Point2D ctrlPs[4]){
	
	int i;
	Point2D ret;
	ret.x = 0; ret.y = 0;
	
	for (i = 0; i < 4; i++){
		double b_t = b_x_t(i, t);
		ret.x += ctrlPs[i].x * b_t;
		ret.y += ctrlPs[i].y * b_t;
	}
	return ret;
	
}

//cubic bernstein polynomial calculation function
double b_x_t(int order, double t){
	double ret;
	
	switch (order){
		case 0:
			ret = -pow(t, 3) + (3 * t*t) - (3*t) + 1;
			break;
		case 1:
			ret = (3 * pow(t, 3)) - (6 * t*t) + (3*t);
			break;
		case 2:
			ret = (-3 * pow(t, 3)) + (3 * t*t);
			break;
		case 3:
			ret = pow(t, 3);
			break;
		default:
			ret = 0;
	}
	
	return ret;
	
}

/*	----------------------------------------------------------
*	Calculate line segments using cubic Polys
*	---------------------------------------------------------
*/

//cubic bernstein polynomial calculation function
//calculates the coeffs a..d
//d=0..a=3
Point2D *PolyCoeffs(Point2D p[4]){
	//Point2D ret[4];
	Point2D *ret = (Point2D*)malloc(sizeof(Point2D) * 4);
	
	ret[3].x = (-p[0].x) + (3*p[1].x) - (3*p[2].x) + p[3].x;
	ret[3].y = (-p[0].y) + (3*p[1].y) - (3*p[2].y) + p[3].y;

	ret[2].x = (3*p[0].x) - (6*p[1].x) + (3*p[2].x);
	ret[2].y = (3*p[0].y) - (6*p[1].y) + (3*p[2].y);

	ret[1].x = (-3*p[0].x) + (3*p[1].x);
	ret[1].y = (-3*p[0].y) + (3*p[1].y);

	ret[0].x = p[0].x;
	ret[0].y = p[0].y;
	
	return ret;
	
}

//cubic 2D Poly calculat0r
//gets an array of the 4 coeffs and param t
//a=3..d=0
//and returns x(t)
Point2D PolyCalculat0r(double t, Point2D coeffs[4]){
	
	int i;
	Point2D ret;
	ret.x = 0; ret.y = 0;
	
	for (i = 0; i < 4; i++){
		ret.x += coeffs[i].x * pow(t,i);
		ret.y += coeffs[i].y * pow(t,i);
	}
	return ret;
	
}

//cubic 2D Poly derivative calculat0r
//gets an array of the 4 coeffs and param t
//a=3..d=0
//and returns x'(t)
Point2D PolyDerivCalculat0r(double t, Point2D coeffs[4]){
	
	int i;
	Point2D ret;
	ret.x = 0; ret.y = 0;
	
	for (i = 1; i < 4; i++){
		ret.x += i * coeffs[i].x * pow(t,i-1);
		ret.y += i * coeffs[i].y * pow(t,i-1);
	}
	return ret;
	
}

//returns a piecewise bezier curve, that is an array of vertices for it 
//this actually returns a quad point, since we also need dx/dy evaluated at each point 
//n is the number of bezier segments, so we have 3N+1 control points from p0-p3N in the array
//n_points is number of points to evaluate along curve, ie. vertices to calculate
//shoud be evenly divisible by n
Quad2D *piecewisePolyCalc(int n, Point2D CtrlPoints[], int n_points){

	//the number of points per line segment, that is each piece of the curve
	int PsPerPiece = n_points / n;
	Point2D currCtrlPs[4];
	Point2D *coeffs;
	
	//Point2D vertices[n_points];
	Quad2D *vertices = (Quad2D*)malloc(sizeof(Quad2D) * n_points);
	
	//the running param u: 0<u<3n
	double u, t, nextX = 0;
	int i,j, nextIdx = 0;
	double a,b,c,d;
	
	//now evaluate each line segment
	for ( i = 0; i < n; i++ ){
		
		currCtrlPs[0] = CtrlPoints[i*3];
		currCtrlPs[1] = CtrlPoints[i*3 + 1];
		currCtrlPs[2] = CtrlPoints[i*3 + 2];
		currCtrlPs[3] = CtrlPoints[i*3 + 3];
		
		//get the coeffs
		coeffs = PolyCoeffs(currCtrlPs);
		
		//and each point along that segment
		for (j = 0; j < PsPerPiece; j++){
			//the running t 0..1
			t = (double) j / PsPerPiece;
			
			//the coord
			vertices[nextIdx].p = PolyCalculat0r(t, coeffs);
			vertices[nextIdx].d = PolyDerivCalculat0r(t, coeffs);
			nextIdx++;
		} 
	}
	
	return vertices;
}


/*	----------------------------------------------------------
*	Calculate line segments using Matrices.. postponed
*	---------------------------------------------------------
*/
