#include "common.h"
#include "control_points.h"

#define N_POINTS 200
#define ANGLE_DELTA 10

TrianglePtr *TableTriangs = NULL;
TrianglePtr *GlassTriangs = NULL;
TrianglePtr *BottleTriangs = NULL;
TrianglePtr *CandleTriangs = NULL;

Material mat;

void make_bezierSurfaces(){
		Point2D *ctrl;
		Color c;
		c.r = c.g = c.b = 1;
		
		if (TableTriangs == NULL){
			cout << "making Table triangs!" << endl;
			ctrl = makeCtrlPoints(TableCtrlPts, N_TABLE_PS);
			c.a = 1;
			
			TableTriangs = calcSurfaceRevolutionTria(N_TABLE_SEGS, ctrl, N_POINTS, ANGLE_DELTA, c);
		}

		if (GlassTriangs == NULL){
			c.a = 0.4;
			cout << "making Wine Glass triangs!" << endl;
			ctrl = makeCtrlPoints(WineGlCPts, N_GLASS_PS);
			GlassTriangs = calcSurfaceRevolutionTria(N_GLASS_SEGS, ctrl, N_POINTS, ANGLE_DELTA, c);
		}

		if (BottleTriangs == NULL){
			c.r = 0;
			c.g = 0;
			c.b = 0;
			c.a = 1;
			cout << "making Wine Bottle triangs!" << endl;
			ctrl = makeCtrlPoints(WineBottleCPts, N_BOTTLE_PS);
			BottleTriangs = calcSurfaceRevolutionTria(N_BOTTLE_SEGS, ctrl, N_POINTS, ANGLE_DELTA, c);
		}

		if (CandleTriangs == NULL){
			c.r = 0.7;
			c.g = 0.7;
			c.b = 0.0;
			c.a = 1;
			cout << "making candle triangs!" << endl;
			ctrl = makeCtrlPoints(CandleCPts, N_CANDLE_PS);
			CandleTriangs = calcSurfaceRevolutionTria(N_CANDLE_SEGS, ctrl, N_POINTS, ANGLE_DELTA, c);
		}
		
}


void drawBottle(){
	glEnable(GL_LIGHTING);              // enable lighting
	glDisable (GL_BLEND);				//disable blending
	
	//set the bottle a little shiny
	mat.setDiffuse(0.5, 0.5, 0.5, 1.0);
	mat.setAmbient(0.5,0.5,0.5,1);
	mat.setSpecular(1,1,1,1,1);
	//set emissive to 0
	mat.setEmissive(0,0,0,1);
	
	draw_bezierSurface(BottleTriangs);
}

void drawCandle(){
	glEnable(GL_LIGHTING);              // enable lighting
	glDisable (GL_BLEND);				//disable blending
	
	mat.setDiffuse(0.5, 0.5, 0.5, 1.0);
	mat.setAmbient(0.5,0.5,0.5,1);
	mat.setSpecular(0,0,0,1,0);
	//set emissive to 0
	mat.setEmissive(0,0,0,1);
	
	draw_bezierSurface(CandleTriangs);
}

void drawTable(){
	glEnable(GL_LIGHTING);              // enable lighting
	glDisable (GL_BLEND);				//disable blending
	
	//set the table a little shiny
	mat.setDiffuse(0.5, 0.5, 0.5, 1.0);
	mat.setAmbient(0,0,0,1);
	mat.setSpecular(1,1,1,1,10);
	//set emissive to 0
	mat.setEmissive(0,0,0,1);
	
	draw_bezierSurface(TableTriangs);
}

void drawWineGlass(){
	float glass_alpha = 0.4;
	//we need to render this a little transparent, this is from OpenGL.org
	//Disable lighting, enable blending, and render your glass geometry with a small alpha value.
	
	glDisable(GL_LIGHTING);              // Disable lighting
	glEnable(GL_BLEND);				//enable blending
	//glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	draw_bezierSurface(GlassTriangs);
	
	//Now, you need to add the specular highlight. 
	//Set your ambient and diffuse material colors to black, and your specular material and light colors to white. 
	//Enable lighting. Set glDepthFunc(GL_EQUAL), then render your glass object a second time.
	mat.setDiffuse(0,0,0,1);
	mat.setAmbient(0,0,0,1);
	mat.setSpecular(1,1,1,1,2);
	glEnable(GL_LIGHTING);              // enable lighting
	glDepthFunc(GL_EQUAL);
	//glDisable(GL_BLEND);				//enable blending
	
	draw_bezierSurface(GlassTriangs);
	
	glDepthFunc(GL_LESS);

	glDisable (GL_BLEND);				//enable blending
	
}

Point2D *makeCtrlPoints(int p[][2], int n){
	
	int i;
	//make some control points for a simple round table
	Point2D *ctrl = (Point2D*)malloc(sizeof(Point2D) * n);
	
	//now assign the points one by one
	for (i = 0; i < n; i++){
		ctrl[i].x = p[i][0];
		ctrl[i].y = p[i][1];
	}
	return ctrl;
}

Point2D *makeCtrlPoints(){
//	return makeCtrlPoints(TableCtrlPts, N_TABLE_PS);
	return makeCtrlPoints(WineGlCPts, N_GLASS_PS);
}


/*
struct TrianglePtr
{
	Vertex3D 	*a,*b,*c;		//pointers to the three vertices forming this triangle
};
*/


void draw_bezierSurface(TrianglePtr *triangs){
	

		//cout << "preparing to draw triangles!" << endl;
		int i;
		int n_triangs = ((360/ANGLE_DELTA) * (N_POINTS-1)) * 2;
		
		//now draw the triangs

		for (i = 0; i < (n_triangs); i++){
			glColor3f(1, 1, 1);
			glBegin(GL_TRIANGLES);
			//cout << "Drawing Triangle#" << i << endl;
			//cout << "Drawing Triangle at (" << triangs[i].a << "," << triangs[i].b << ", "<< triangs[i].c << ")" << endl;
			
			glColor4f((*triangs[i].a).color.r, (*triangs[i].a).color.g, (*triangs[i].a).color.b, (*triangs[i].a).color.a);
			glTexCoord2d((*triangs[i].a).u, (*triangs[i].a).v);
			glNormal3f((*triangs[i].a).normal[0], (*triangs[i].a).normal[1], (*triangs[i].a).normal[2]);
			glVertex3d((*triangs[i].a).coord[0], (*triangs[i].a).coord[1], (*triangs[i].a).coord[2]);
			
			glColor4f((*triangs[i].b).color.r, (*triangs[i].b).color.g, (*triangs[i].b).color.b, (*triangs[i].b).color.a);
			glTexCoord2d((*triangs[i].b).u, (*triangs[i].b).v);
			glNormal3f((*triangs[i].b).normal[0], (*triangs[i].b).normal[1], (*triangs[i].b).normal[2]);
			glVertex3d((*triangs[i].b).coord[0], (*triangs[i].b).coord[1], (*triangs[i].b).coord[2]);
			
			glColor4f((*triangs[i].c).color.r, (*triangs[i].c).color.g, (*triangs[i].c).color.b, (*triangs[i].c).color.a);
			glTexCoord2d((*triangs[i].c).u, (*triangs[i].c).v);
			glNormal3f((*triangs[i].c).normal[0], (*triangs[i].c).normal[1], (*triangs[i].c).normal[2]);
			glVertex3d((*triangs[i].c).coord[0], (*triangs[i].c).coord[1], (*triangs[i].c).coord[2]);
			glEnd();


			/*
			//now draw the vertices in green
			glBegin(GL_POINTS);
			glColor3f(0, 1, 0);
	
			glVertex3d((*triangs[i].a).coord[0], (*triangs[i].a).coord[1], (*triangs[i].a).coord[2]);
			glVertex3d((*triangs[i].b).coord[0], (*triangs[i].b).coord[1], (*triangs[i].b).coord[2]);
			glVertex3d((*triangs[i].c).coord[0], (*triangs[i].c).coord[1], (*triangs[i].c).coord[2]);
			
			glEnd();
			*/
		}

	
}